"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var path_1 = require("path");
var sysWrapper_1 = require("../utils/sysWrapper");
var smartModel_1 = require("../models/smartModel");
var SmartApiSwaggerYamlModels = (function (_super) {
    tslib_1.__extends(SmartApiSwaggerYamlModels, _super);
    function SmartApiSwaggerYamlModels(name, chaincodeName, projectName, modelNames, modelClasses, methods, ignoreConvention) {
        var _this = _super.call(this, name, projectName) || this;
        _this.name = name;
        _this.chaincodeName = chaincodeName;
        _this.projectName = projectName;
        _this.modelNames = modelNames;
        _this.modelClasses = modelClasses;
        _this.methods = methods;
        _this.ignoreConvention = ignoreConvention;
        return _this;
    }
    SmartApiSwaggerYamlModels.prototype.recompile = function () {
        throw new Error('Method not implemented.');
    };
    SmartApiSwaggerYamlModels.prototype.save = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.dto = this.getDTO();
                        console.log("DTO==" + this.dto);
                        return [4, sysWrapper_1.SysWrapper.createFileFromTemplate(this.filePath, {
                                dto: this.dto
                            }, this.templateFile)];
                    case 1:
                        _a.sent();
                        return [2];
                }
            });
        });
    };
    Object.defineProperty(SmartApiSwaggerYamlModels.prototype, "chaincodeClientFolder", {
        get: function () {
            return this.chaincodeName.match(/[a-z]+/gi)
                .map(function (word) {
                return word + '-cc/client';
            })
                .join('');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SmartApiSwaggerYamlModels.prototype, "applicationName", {
        get: function () {
            return this.chaincodeName.match(/[a-z]+/gi)
                .map(function (word) {
                return word + '-app';
            })
                .join('');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SmartApiSwaggerYamlModels.prototype, "templateFile", {
        get: function () {
            return path_1.join(__dirname, '../../templates/_smartApiSwagger.yaml.ejs');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SmartApiSwaggerYamlModels.prototype, "filePath", {
        get: function () {
            return "packages/" + this.applicationName + "/server/common/swagger/Api.yaml";
        },
        enumerable: true,
        configurable: true
    });
    SmartApiSwaggerYamlModels.prototype.getDTO = function () {
        var dto = {};
        dto.projectName = this.projectName;
        dto.models = [];
        dto.serviceMethods = [];
        for (var _i = 0, _a = this.modelClasses; _i < _a.length; _i++) {
            var modelClass = _a[_i];
            var modelObj = {};
            ;
            modelObj.modelName = modelClass.getStructure().name;
            modelObj.modelNameLowered = modelObj.modelName.charAt(0).toLowerCase() + modelObj.modelName.slice(1);
            modelObj.modelProperties = [];
            var modelPropertiesNames = [];
            var baseClass = modelClass.getBaseClass();
            if (baseClass != undefined) {
                var baseClassProperties = baseClass.getProperties();
                for (var _b = 0, baseClassProperties_1 = baseClassProperties; _b < baseClassProperties_1.length; _b++) {
                    var property = baseClassProperties_1[_b];
                    if (modelPropertiesNames.indexOf(property.getName()) >= 0) {
                        continue;
                    }
                    var modelPropertyObj = {};
                    modelPropertyObj.propName = property.getName();
                    if (property.getType().getText() == undefined || property.getType().getText() == 'string') {
                        modelPropertyObj.propType = 'string';
                        modelPropertyObj.propExample = 'a_text';
                    }
                    else if (property.getType().getText() == 'number') {
                        modelPropertyObj.propType = 'number';
                        modelPropertyObj.propExample = '123';
                    }
                    else {
                        modelPropertyObj.propType = property.getType().getText();
                        modelPropertyObj.propExample = 'null';
                    }
                    modelObj.modelProperties.push(modelPropertyObj);
                    modelPropertiesNames.push(property.getName());
                }
            }
            for (var _c = 0, _d = modelClass.getStructure().properties; _c < _d.length; _c++) {
                var property = _d[_c];
                if (modelPropertiesNames.indexOf(property.name) >= 0) {
                    continue;
                }
                var modelPropertyObj = {};
                ;
                modelPropertyObj.propName = property.name;
                if (property.type == undefined || property.type == 'string') {
                    modelPropertyObj.propType = 'string';
                    modelPropertyObj.propExample = 'a_text';
                }
                else if (property.type == 'number') {
                    modelPropertyObj.propType = 'number';
                    modelPropertyObj.propExample = '123';
                }
                else {
                    modelPropertyObj.propType = property.type;
                    modelPropertyObj.propExample = 'null';
                }
                modelObj.modelProperties.push(modelPropertyObj);
                modelPropertiesNames.push(property.name);
            }
            dto.models.push(modelObj);
        }
        var _loop_1 = function (method) {
            if (method.getDecorator("Service")) {
                var serviceObj = {};
                serviceObj.methodName = method.getName();
                serviceObj.methodEndPoint = serviceObj.methodName;
                var parameters_1 = [];
                var parametersForEndpoint_1 = "";
                var first_1 = true;
                method.getParameters().forEach(function (parameter) {
                    var param = {};
                    ;
                    param.name = parameter.getName();
                    param.type = parameter.getSymbol().getName();
                    parameters_1.push(param);
                    if (first_1) {
                        parametersForEndpoint_1 = "/" + "{" + parameter.getName() + "}";
                        first_1 = false;
                    }
                    else {
                        parametersForEndpoint_1 = parametersForEndpoint_1.concat("/{").concat(parameter.getName()) + "}";
                    }
                });
                serviceObj.methodEndPoint = serviceObj.methodEndPoint + parametersForEndpoint_1;
                serviceObj.methodParameters = parameters_1;
                dto.serviceMethods.push(serviceObj);
            }
        };
        for (var _e = 0, _f = this.methods; _e < _f.length; _e++) {
            var method = _f[_e];
            _loop_1(method);
        }
        return dto;
    };
    return SmartApiSwaggerYamlModels;
}(smartModel_1.SmartModel));
exports.SmartApiSwaggerYamlModels = SmartApiSwaggerYamlModels;
//# sourceMappingURL=smartApiSwaggerYaml.smart-model.js.map