"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var path_1 = require("path");
var sysWrapper_1 = require("../utils/sysWrapper");
var smartModel_1 = require("../models/smartModel");
var SmartRouterModels = (function (_super) {
    tslib_1.__extends(SmartRouterModels, _super);
    function SmartRouterModels(name, chaincodeName, projectName, methods, controllerClassName, ignoreConvention) {
        var _this = _super.call(this, name, projectName) || this;
        _this.name = name;
        _this.chaincodeName = chaincodeName;
        _this.projectName = projectName;
        _this.methods = methods;
        _this.controllerClassName = controllerClassName;
        _this.ignoreConvention = ignoreConvention;
        return _this;
    }
    SmartRouterModels.prototype.recompile = function () {
        throw new Error('Method not implemented.');
    };
    SmartRouterModels.prototype.save = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, sysWrapper_1.SysWrapper.createFileFromTemplate(this.filePath, {
                            dto: this.getDTO()
                        }, this.templateFile)];
                    case 1:
                        _a.sent();
                        return [2];
                }
            });
        });
    };
    Object.defineProperty(SmartRouterModels.prototype, "chaincodeClientFolder", {
        get: function () {
            return this.chaincodeName.match(/[a-z]+/gi)
                .map(function (word) {
                return word + '-cc/client';
            })
                .join('');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SmartRouterModels.prototype, "applicationName", {
        get: function () {
            return this.chaincodeName.match(/[a-z]+/gi)
                .map(function (word) {
                return word + '-app';
            })
                .join('');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SmartRouterModels.prototype, "templateFile", {
        get: function () {
            return path_1.join(__dirname, '../../templates/_smartRouter.ts.ejs');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SmartRouterModels.prototype, "filePath", {
        get: function () {
            return this.projectRoot + "/packages/" + this.applicationName + "/server/api/controllers/examples/router.ts";
        },
        enumerable: true,
        configurable: true
    });
    SmartRouterModels.prototype.getDTO = function () {
        var dto = {};
        var getAllMethods = [];
        var getByIdMethods = [];
        var createMethods = [];
        var serviceMethods = [];
        var _loop_1 = function (method) {
            if (method.getDecorator("GetAll")) {
                var getAllObj = {};
                getAllObj.methodName = method.getName();
                getAllObj.methodClassName = method.getDecorator("GetAll").getArguments()[0].getText().replace(/[ '|\" ]/g, '');
                getAllObj.methodEndPoint = getAllObj.methodClassName.charAt(0).toLowerCase() + getAllObj.methodClassName.slice(1) + 's';
                getAllMethods.push(getAllObj);
            }
            else if (method.getDecorator("GetById")) {
                var getIdObj = {};
                getIdObj.methodName = method.getName();
                getIdObj.methodClassName = method.getDecorator("GetById").getArguments()[0].getText().replace(/[ '|\" ]/g, '');
                getIdObj.methodEndPoint = getIdObj.methodClassName.charAt(0).toLowerCase() + getIdObj.methodClassName.slice(1) + 's';
                getByIdMethods.push(getIdObj);
            }
            else if (method.getDecorator("Create")) {
                var createObj = {};
                createObj.methodName = method.getName();
                createObj.methodClassName = method.getDecorator("Create").getArguments()[0].getText().replace(/[ '|\" ]/g, '');
                createObj.methodEndPoint = createObj.methodClassName.charAt(0).toLowerCase() + createObj.methodClassName.slice(1) + 's';
                createMethods.push(createObj);
            }
            else if (method.getDecorator("Service")) {
                var serviceObj = {};
                serviceObj.methodName = method.getName();
                serviceObj.methodEndPoint = serviceObj.methodName;
                var parameters_1 = "";
                var first_1 = true;
                method.getParameters().forEach(function (parameter) {
                    if (first_1) {
                        parameters_1 = "/" + ":" + parameter.getName();
                        first_1 = false;
                    }
                    else {
                        parameters_1 = parameters_1.concat("/:").concat(parameter.getName());
                    }
                });
                serviceObj.methodEndPoint = serviceObj.methodEndPoint + parameters_1;
                console.log("methodEndPoint = " + serviceObj.methodEndPoint);
                serviceMethods.push(serviceObj);
            }
        };
        for (var _i = 0, _a = this.methods; _i < _a.length; _i++) {
            var method = _a[_i];
            _loop_1(method);
        }
        dto.controllerClassName = this.controllerClassName;
        dto.getAllMethods = getAllMethods;
        dto.getByIdMethods = getByIdMethods;
        dto.createMethods = createMethods;
        dto.serviceMethods = serviceMethods;
        return dto;
    };
    return SmartRouterModels;
}(smartModel_1.SmartModel));
exports.SmartRouterModels = SmartRouterModels;
//# sourceMappingURL=smartRouter.smart-model.js.map