"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var path_1 = require("path");
var ts_simple_ast_1 = require("ts-simple-ast");
var ReflectionUtils;
(function (ReflectionUtils) {
    function getClassNames(pathPattern) {
        return new Promise(function (fulfilled, rejected) {
            var classNames = [];
            var project = new ts_simple_ast_1.Project({});
            project.addExistingSourceFiles(pathPattern);
            var sourceFiles = project.getSourceFiles();
            for (var _i = 0, sourceFiles_1 = sourceFiles; _i < sourceFiles_1.length; _i++) {
                var sourceFile = sourceFiles_1[_i];
                classNames.push(sourceFile.getClasses()[0].getStructure().name);
            }
            fulfilled(classNames);
        });
    }
    ReflectionUtils.getClassNames = getClassNames;
    function getFirstClass(pathPattern) {
        return new Promise(function (fulfilled, rejected) {
            var classes = [];
            var project = new ts_simple_ast_1.Project({});
            project.addExistingSourceFiles(pathPattern);
            var sourceFiles = project.getSourceFiles();
            for (var _i = 0, sourceFiles_2 = sourceFiles; _i < sourceFiles_2.length; _i++) {
                var sourceFile = sourceFiles_2[_i];
                classes.push(sourceFile.getClasses()[0]);
            }
            fulfilled(classes[0]);
        });
    }
    ReflectionUtils.getFirstClass = getFirstClass;
    function getImportDeclarations(pathPattern) {
        return new Promise(function (fulfilled, rejected) {
            var imports = [];
            var project = new ts_simple_ast_1.Project({});
            project.addExistingSourceFiles(pathPattern);
            var sourceFiles = project.getSourceFiles();
            for (var _i = 0, sourceFiles_3 = sourceFiles; _i < sourceFiles_3.length; _i++) {
                var sourceFile = sourceFiles_3[_i];
                Array.prototype.push.apply(imports, sourceFile.getImportDeclarations());
            }
            fulfilled(imports);
        });
    }
    ReflectionUtils.getImportDeclarations = getImportDeclarations;
    function getClassImportDeclarations(pathPattern, className) {
        return new Promise(function (fulfilled, rejected) {
            var imports = [];
            var project = new ts_simple_ast_1.Project({});
            project.addExistingSourceFiles(pathPattern);
            var sourceFiles = project.getSourceFiles();
            for (var _i = 0, sourceFiles_4 = sourceFiles; _i < sourceFiles_4.length; _i++) {
                var sourceFile = sourceFiles_4[_i];
                var sourceFileClasses = sourceFile.getClasses();
                for (var _a = 0, sourceFileClasses_1 = sourceFileClasses; _a < sourceFileClasses_1.length; _a++) {
                    var sourceFileClass = sourceFileClasses_1[_a];
                    if (sourceFileClass.getName() == className) {
                        Array.prototype.push.apply(imports, sourceFile.getImportDeclarations());
                    }
                }
            }
            fulfilled(imports);
        });
    }
    ReflectionUtils.getClassImportDeclarations = getClassImportDeclarations;
    function getClass(pathPattern, className) {
        return new Promise(function (fulfilled, rejected) {
            var classes = [];
            var project = new ts_simple_ast_1.Project({});
            project.addExistingSourceFiles(pathPattern);
            var sourceFiles = project.getSourceFiles();
            for (var _i = 0, sourceFiles_5 = sourceFiles; _i < sourceFiles_5.length; _i++) {
                var sourceFile = sourceFiles_5[_i];
                Array.prototype.push.apply(classes, sourceFile.getClasses());
            }
            for (var i in classes) {
                if (classes[i].getName().indexOf(className) >= 0) {
                    return fulfilled(classes[i]);
                }
            }
            return fulfilled(null);
        });
    }
    ReflectionUtils.getClass = getClass;
    function getClassFromSource(sourceFile, className) {
        return new Promise(function (fulfilled, rejected) {
            var classes = sourceFile.getClasses();
            for (var i in classes) {
                if (classes[i].getName().indexOf(className) >= 0) {
                    return fulfilled(classes[i]);
                }
            }
            return fulfilled(null);
        });
    }
    ReflectionUtils.getClassFromSource = getClassFromSource;
    function getClasses(pathPattern) {
        return new Promise(function (fulfilled, rejected) {
            var classes = [];
            var project = new ts_simple_ast_1.Project({});
            project.addExistingSourceFiles(pathPattern);
            var sourceFiles = project.getSourceFiles();
            for (var _i = 0, sourceFiles_6 = sourceFiles; _i < sourceFiles_6.length; _i++) {
                var sourceFile = sourceFiles_6[_i];
                Array.prototype.push.apply(classes, sourceFile.getClasses());
            }
            fulfilled(classes);
        });
    }
    ReflectionUtils.getClasses = getClasses;
    function getClassMethods(pathPattern, className) {
        return new Promise(function (fulfilled, rejected) {
            var methods = [];
            var project = new ts_simple_ast_1.Project({});
            project.addExistingSourceFiles(pathPattern);
            var sourceFiles = project.getSourceFiles();
            var sourceFile = sourceFiles[0];
            var classClass = sourceFile.getClassOrThrow(className);
            methods = classClass.getMethods();
            fulfilled(methods);
        });
    }
    ReflectionUtils.getClassMethods = getClassMethods;
    function getClassParameters(pathPattern, className) {
        return new Promise(function (fulfilled, rejected) {
            var properties = [];
            var project = new ts_simple_ast_1.Project({});
            project.addExistingSourceFiles(pathPattern);
            var sourceFiles = project.getSourceFiles();
            var sourceFile = sourceFiles[0];
            var classClass = sourceFile.getClassOrThrow(className);
            properties = classClass.getStructure().properties;
            for (var _i = 0, properties_1 = properties; _i < properties_1.length; _i++) {
                var prop = properties_1[_i];
                var propType = void 0;
                if (prop.type == undefined) {
                    propType = 'string';
                }
                else {
                    propType = prop.type;
                }
            }
            fulfilled(properties);
        });
    }
    ReflectionUtils.getClassParameters = getClassParameters;
    function getClassParametersFull(pathPattern, className, fullClassProperties) {
        return new Promise(function (fulfilled, rejected) {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var classClass, classClassStructure, baseClassName, importDeclarations, _i, importDeclarations_1, importDeclaration, moduleSpecifier, baseClassSource, returnArray, _a, _b;
                return tslib_1.__generator(this, function (_c) {
                    switch (_c.label) {
                        case 0: return [4, getClass(pathPattern, className)];
                        case 1:
                            classClass = _c.sent();
                            classClassStructure = classClass.getStructure();
                            fullClassProperties = classClassStructure.properties;
                            if (!(classClassStructure.extends == undefined)) return [3, 2];
                            return [2, fulfilled(fullClassProperties)];
                        case 2:
                            baseClassName = classClassStructure.extends.substring(0, classClassStructure.extends.indexOf("<"));
                            importDeclarations = [];
                            return [4, ReflectionUtils.getClassImportDeclarations(pathPattern, className)];
                        case 3:
                            importDeclarations = _c.sent();
                            _i = 0, importDeclarations_1 = importDeclarations;
                            _c.label = 4;
                        case 4:
                            if (!(_i < importDeclarations_1.length)) return [3, 7];
                            importDeclaration = importDeclarations_1[_i];
                            if (!(importDeclaration.getText().indexOf(baseClassName) >= 0)) return [3, 6];
                            moduleSpecifier = importDeclaration.getModuleSpecifierValue();
                            baseClassSource = path_1.join(__dirname, ".") + "/../../node_modules/" + moduleSpecifier + "/**/*.ts*";
                            _b = (_a = fullClassProperties).concat;
                            return [4, getClassParametersFull(baseClassSource, baseClassName, fullClassProperties)];
                        case 5:
                            returnArray = _b.apply(_a, [_c.sent()]);
                            return [2, fulfilled(removeDuplicates(returnArray))];
                        case 6:
                            _i++;
                            return [3, 4];
                        case 7: return [2];
                    }
                });
            });
        });
    }
    ReflectionUtils.getClassParametersFull = getClassParametersFull;
    function removeDuplicates(arr) {
        var unique_array = [];
        var unique_names_array = [];
        for (var i = 0; i < arr.length; i++) {
            if (unique_names_array.indexOf(arr[i].name) == -1) {
                unique_names_array.push(arr[i].name);
                unique_array.push(arr[i]);
            }
        }
        return unique_array;
    }
})(ReflectionUtils = exports.ReflectionUtils || (exports.ReflectionUtils = {}));
//# sourceMappingURL=reflectionUtils.js.map