"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var memFs = require("mem-fs");
var memFsEditor = require("mem-fs-editor");
var ejs = require("ejs");
var fs = require("fs-extra");
var shelljs_1 = require("shelljs");
var path_1 = require("path");
var child = require("child_process");
var SysWrapper;
(function (SysWrapper) {
    function createFile(filePath, contents) {
        return new Promise(function (fulfilled, rejected) {
            try {
                write(filePath, contents, fulfilled);
            }
            catch (ex) {
                rejected(ex);
            }
        }).then(function () { showSuccessInfo(filePath); });
    }
    SysWrapper.createFile = createFile;
    function createFileFromTemplate(filePath, contents, templatePath) {
        return renderTemplateFromFile(templatePath, contents).then(function (compiledContents) {
            return new Promise(function (fulfilled, rejected) {
                try {
                    write(filePath, compiledContents, fulfilled);
                }
                catch (ex) {
                    rejected(ex);
                }
            }).then(function () { showSuccessInfo(filePath); });
        });
    }
    SysWrapper.createFileFromTemplate = createFileFromTemplate;
    function createFileRaw(filePath, contents) {
        return new Promise(function (fulfilled, rejected) {
            try {
                writeBuffer(filePath, contents, fulfilled);
            }
            catch (ex) {
                rejected(ex);
            }
        });
    }
    SysWrapper.createFileRaw = createFileRaw;
    function removePath(filePath) {
        return remove(filePath);
    }
    SysWrapper.removePath = removePath;
    function getFile(filePath, content) {
        return new Promise(function (fulfilled, rejected) {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var _a, store, editor, file;
                return tslib_1.__generator(this, function (_b) {
                    switch (_b.label) {
                        case 0:
                            if (!content) return [3, 2];
                            _a = fulfilled;
                            return [4, renderTemplateFromFile(filePath, content)];
                        case 1:
                            _a.apply(void 0, [_b.sent()]);
                            return [3, 3];
                        case 2:
                            store = memFs.create();
                            editor = memFsEditor.create(store);
                            try {
                                file = editor.read(filePath);
                                if (!file) {
                                    rejected('Empty or not found file.');
                                }
                                fulfilled(file);
                            }
                            catch (ex) {
                                rejected(ex);
                            }
                            _b.label = 3;
                        case 3: return [2];
                    }
                });
            });
        });
    }
    SysWrapper.getFile = getFile;
    function execFile(filePath, content) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var renderedFileContent, simpleFileContent;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!content) return [3, 2];
                        return [4, renderTemplateFromFile(filePath, content)];
                    case 1:
                        renderedFileContent = _a.sent();
                        return [2, shelljs_1.exec(renderedFileContent, { silent: false })];
                    case 2: return [4, getFile(filePath)];
                    case 3:
                        simpleFileContent = _a.sent();
                        return [2, shelljs_1.exec(simpleFileContent, { silent: false })];
                }
            });
        });
    }
    SysWrapper.execFile = execFile;
    function getFileRaw(filePath) {
        return new Promise(function (fulfilled, rejected) {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var store, editor, file;
                return tslib_1.__generator(this, function (_a) {
                    try {
                        store = memFs.create();
                        editor = memFsEditor.create(store);
                        file = editor.read(filePath, { raw: true });
                        if (!file) {
                            rejected('Empty or not found file.');
                        }
                        fulfilled(file);
                    }
                    catch (ex) {
                        rejected(ex);
                    }
                    return [2];
                });
            });
        });
    }
    SysWrapper.getFileRaw = getFileRaw;
    function copyFile(from, to) {
        console.log("copy from: " + from);
        console.log("copy to: " + to);
        return new Promise(function (fulfilled, rejected) {
            var store = memFs.create();
            var editor = memFsEditor.create(store);
            editor.copy(from, to);
            editor.commit([], fulfilled);
        });
    }
    SysWrapper.copyFile = copyFile;
    function existsPath(filePath) {
        return new Promise(function (fulfilled, rejected) {
            var store = memFs.create();
            var editor = memFsEditor.create(store);
            fulfilled(editor.exists(filePath));
        });
    }
    SysWrapper.existsPath = existsPath;
    function createJSON(filePath, contents) {
        return new Promise(function (fulfilled, rejected) {
            var store = memFs.create();
            var editor = memFsEditor.create(store);
            editor.writeJSON(filePath, contents);
            editor.commit([], fulfilled);
        });
    }
    SysWrapper.createJSON = createJSON;
    function getJSON(filePath) {
        return new Promise(function (fulfilled, rejected) {
            try {
                var store = memFs.create();
                var editor = memFsEditor.create(store);
                fulfilled(editor.readJSON(filePath));
            }
            catch (ex) {
                rejected(ex);
            }
        });
    }
    SysWrapper.getJSON = getJSON;
    function createFolder(folder) {
        return fs.ensureDir(folder);
    }
    SysWrapper.createFolder = createFolder;
    function renderTemplateFromFile(filePath, content) {
        return new Promise(function (fulfilled, rejected) {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var store, editor, file, _a;
                return tslib_1.__generator(this, function (_b) {
                    switch (_b.label) {
                        case 0:
                            store = memFs.create();
                            editor = memFsEditor.create(store);
                            file = editor.read(filePath);
                            if (!file) {
                                return [2, rejected(Error(filePath + " does not exist."))];
                            }
                            _a = fulfilled;
                            return [4, renderTemplateFromContent(file, content)];
                        case 1:
                            _a.apply(void 0, [_b.sent()]);
                            return [2];
                    }
                });
            });
        });
    }
    SysWrapper.renderTemplateFromFile = renderTemplateFromFile;
    function renderTemplateFromContent(templateContent, content) {
        return new Promise(function (fulfilled, rejected) {
            var renderedFile = ejs.render(templateContent, content);
            fulfilled(renderedFile);
        });
    }
    SysWrapper.renderTemplateFromContent = renderTemplateFromContent;
    function remove(filePath) {
        return fs.remove(filePath);
    }
    function write(filePath, contents, cb) {
        var store = memFs.create();
        var editor = memFsEditor.create(store);
        editor.write(filePath, contents);
        editor.commit([], cb);
    }
    function writeBuffer(filePath, contents, cb) {
        var store = memFs.create();
        var editor = memFsEditor.create(store);
        editor.write(filePath, contents);
        editor.commit([], cb);
    }
    function showSuccessInfo(filePath) {
        console.log("CREATE " + path_1.relative('', filePath) + " (" + fs.statSync(filePath).size + " bytes)");
    }
    function enumFilesInFolder(folder) {
        return new Promise(function (fulfilled, rejected) {
            fs.readdir(path_1.relative('', folder), function (err, files) {
                fulfilled(files);
            });
        });
    }
    SysWrapper.enumFilesInFolder = enumFilesInFolder;
    function enumFilesInFolderFiltered(folder, pattern) {
        return new Promise(function (fulfilled, rejected) {
            fs.readdir(path_1.relative('', folder), function (err, files) {
                var filteredFiles = [];
                for (var i in files) {
                    if (files[i].search(new RegExp(pattern)) >= 0) {
                        filteredFiles.push(files[i]);
                    }
                }
                fulfilled(filteredFiles);
            });
        });
    }
    SysWrapper.enumFilesInFolderFiltered = enumFilesInFolderFiltered;
    function executeCommand(command, tags) {
        return new Promise(function (fulfilled, rejected) {
            var _this = this;
            var commandExec = child.spawn(command, tags, { stdio: [process.stdin, process.stdout, process.stderr] });
            commandExec.on('close', function (code) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                return tslib_1.__generator(this, function (_a) {
                    fulfilled("ok");
                    return [2];
                });
            }); });
        });
    }
    SysWrapper.executeCommand = executeCommand;
})(SysWrapper = exports.SysWrapper || (exports.SysWrapper = {}));
//# sourceMappingURL=sysWrapper.js.map